from bevy import Constructor
import bevy.config.config as config
from typing import Any, Optional


class SectionLoader:
    def __init__(self, section_name: str, filename: Optional[str] = None):
        self.section_name = section_name
        self.filename = filename

    def __bevy_build__(self, bevy_constructor: Constructor, *args, **kwargs) -> Any:
        config_manager = bevy_constructor.get(config.Config)
        return config_manager.get_section(self.section_name, self.filename)



from bevy.config.loader import Loader
from typing import Any


class ConfigFile:
    def __init__(self, loader: Loader):
        self._loader = loader
        self._data = None

    @property
    def config_data(self) -> dict[str, Any]:
        if not self._data:
            self._data = self._loader.load()

        return self._data

    def get_section(self, name: str) -> Any:
        return self.config_data.get(name)



from bevy.config.loader import Loader
from bevy.config.config_file import ConfigFile
from bevy.config.exceptions import NoLoadersRegistered, NoResolversRegistered
from bevy.config.resolver import Resolver, Reader
from bevy.config.section_loader import SectionLoader
from typing import Optional, Sequence, Type


class Config:
    def __init__(
        self,
        default_filename: str = "config",
        loaders: Sequence[Type[Loader]] = tuple(),
        resolvers: Sequence[Resolver] = tuple(),
    ):
        self._default_filename = default_filename
        self._loaders = set(loaders)
        self._resolvers = set(resolvers)
        self._supported_file_types = tuple()

    @property
    def supported_file_types(self) -> tuple[str]:
        """A tuple of file extensions that the config has loaders for."""
        if not self._supported_file_types:
            self._supported_file_types = sum(
                (loader.file_types for loader in self._loaders), tuple()
            )
        return self._supported_file_types

    def get_config_file(self, filename: Optional[str] = None) -> ConfigFile:
        """Searches all resolvers for a file matching the filename and returns a ConfigFile object for accessing
        sections. If no filename is given it will fallback to using the default set on the Config object, this is often
        just "config". The filename should not contain an extension, the extension will be determined by the loaded
        loaders and by what the resolvers are able to find.

        This function will raise:
        - bevy.extensions.config.exceptions.NoLoadersRegistered
          If no loaders have been registered.
        - bevy.extensions.config.exceptions.NoResolversRegistered
          If no resolvers have been registered.
        - FileNotFound
          If no resolver is able to find a matching config file with a supported extension.
        - ValueError
          If no loaders match the file type given by the Reader returned by the resolvers.
        """
        if not self._loaders:
            raise NoLoadersRegistered(
                "You must register a file loader in order to load config files"
            )

        if not self._resolvers:
            raise NoResolversRegistered(
                "You must register a file resolver in order to find config files"
            )

        filename = filename if filename else self._default_filename
        reader = self._find_file_reader(filename)
        loader = self._find_file_type_loader(reader)
        return ConfigFile(loader)

    def get_section(self, section_name: str, filename: Optional[str] = None):
        config = self.get_config_file(filename)
        return config.get_section(section_name)

    def add_resolver(self, resolver: Resolver):
        self._resolvers.add(resolver)

    def remove_resolver(self, resolver: Resolver):
        self._resolvers.remove(resolver)

    def add_loader(self, loader: Loader):
        """Adds a loader to the config manager and invalidates the supported file types cache."""
        self._loaders.add(loader)
        self._supported_file_types = tuple()

    def remove_loader(self, loader: Loader):
        """Removes a loader from the config manager and invalidates the supported file types cache."""
        self._loaders.remove(loader)
        self._supported_file_types = tuple()

    def _find_file_reader(self, filename: str) -> Reader:
        for resolver in self._resolvers:
            reader = resolver.get_file_reader(filename, self.supported_file_types)
            if reader:
                return reader

        raise FileNotFoundError(
            f"Could not find find a file that matched {filename!r}, looked for these file types: "
            f"{', '.join(self.supported_file_types) if self.supported_file_types else 'No file types set by loaders'}"
        )

    def _find_file_type_loader(self, reader: Reader) -> Loader:
        for loader in self._loaders:
            if reader.file_type in loader.file_types:
                return loader(reader)

        raise ValueError(
            f"Could not find a loader that matched the file type {reader.file_type!r}, looked for these file types: "
            f"{', '.join(self.supported_file_types) if self.supported_file_types else 'No file types set by loaders'}"
        )

    def __class_getitem__(cls, item) -> SectionLoader:
        section = item[0] if isinstance(item, tuple) else item
        filename = item[1] if len(item) == 2 else None
        return SectionLoader(section, filename)



from bevy.config.resolver import Reader
from typing import Any, Protocol, runtime_checkable


@runtime_checkable
class Loader(Protocol):
    file_types: tuple[str]

    def __init__(self, reader: Reader):
        ...

    def load(self) -> dict[str, Any]:
        ...

    def save(self, data: dict[str, Any]):
        ...


def is_loader(obj):
    return isinstance(obj, Loader)



from typing import Optional, Protocol, runtime_checkable


@runtime_checkable
class Reader(Protocol):
    file_type: str

    def read(self) -> str:
        ...

    def save(self, data: str):
        ...


def is_reader(obj):
    return isinstance(obj, Reader)


@runtime_checkable
class Resolver(Protocol):
    def get_file_reader(
        self, filename: str, file_types: tuple[str]
    ) -> Optional[Reader]:
        ...


def is_resolver(obj):
    return isinstance(obj, Resolver)



from pathlib import Path
from typing import Union


class FileReader:
    def __init__(self, file_path: Union[Path, str]):
        self.file_path = Path(file_path)

        if not self.file_path.is_file():
            raise ValueError(
                f"The file reader must be given a path to a file, got {file_path!r}"
            )

    @property
    def file_type(self) -> str:
        return self.file_path.suffix[1:]

    def read(self) -> str:
        with self.file_path.open() as file:
            return file.read()

    def save(self, data: str):
        with self.file_path.open("w") as file:
            file.write(data)



from bevy.config.file_reader import FileReader
from pathlib import Path
from typing import Optional, Union


class DirectoryResolver:
    def __init__(self, directory: Union[Path, str]):
        self.directory = Path(directory)

        if not self.directory.is_dir():
            self.directory = self.directory.parent

    def get_file_reader(
        self, filename: str, file_types: tuple[str]
    ) -> Optional[FileReader]:
        for item in self.directory.iterdir():
            if item.is_file() and item.suffix[1:] in file_types:
                return FileReader(item)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['config']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'bevy.config',
    'version': '0.1.1',
    'description': 'A config loader designed to be used with the Bevy dependency injection framework.',
    'long_description': '# Bevy\nBevy makes using *Dependency Injection* a breeze so that you can focus on creating amazing code.\n\n## Installation\n```shell script\npip install bevy\n```\n\n**[Documentation](docs/documentation.md)**\n\n## Dependency Injection\nPut simply, *Dependency Injection* is a design pattern where the objects that your class depends on are instantiated outside of the class. Those dependencies are then injected into your class when it is instantiated.\nThis promotes loosely coupled code where your class doesn’t require direct knowledge of what classes it depends on or how to create them. Instead your class declares what class interface it expects and an outside framework handles the work of creating the class instances with the correct interface.\n\n## Interfaces\nPython doesn’t have an actual interface implementation like many other languages. Class inheritance, however, can be used in a very similar way since sub classes will likely have the same fundamental interface as their base class. \n\n## Why Do I Care?\n*Dependency Injection* and its reliance on abstract interfaces makes your code easier to maintain:\n- Changes can be made without needing to alter implementation details in unrelated code, so long as the interface isn’t modified in a substantial way.\n- Tests can provide mock implementations of dependencies without needing to jump through hoops to inject them. They can provide the mock to the context and Bevy will make sure it is used where appropriate.\n\n## How It Works\nBevy is an object oriented *dependency injection* framework. Similar to Pydantic, it relies on Python 3\'s class\nannotations, using them to determine what dependencies a class has.\n\n**Example**\n```py\n@bevy.injectable\nclass Example:\n    dependency: Dependency\n```\nEach dependency when instantiated is added to a context repository for reuse. This allows many classes to share the same\ninstance of each dependency. This is handy for sharing things like database connections, config files, or authenticated\nAPI sessions.\n\n## Bevy Constructors\n\nTo instantiate classes and have Bevy inject their dependencies it is necessary to use a\n[`bevy.Constructor`](#Constructor). The constructor takes a [`bevy.Injectable`](#Injectable) and any args necessary to\ninstantiate it. Calling [`Constructor.build`](#Constructor.build) on the constructor will then create an instance of the\n`Injectable` with all dependencies injected.\n\n**Example**\n```py\nconstructor = bevy.Constructor(Example)\nexample = constructor.build()\n```\n### Configuring Dependencies\n\nWhen the `Constructor` encounters a dependency that is not in the context repository it will attempt to create the\ndependency. The approach is very naive, it will just call the dependency with no arguments. If it succeeds it will be\nadded to the repository for later use and injected into the class.\n\nThis behavior can be changed by passing an instantiated dependency to [`Constructor.add`](#Constructor.add).\n**Example**\n```py\nconstructor.add(Dependency("foobar"))\n```\nWhen adding an `Injectable` it is necessary to use [`Constructor.branch`](#Constructor.branch) as it will inherit all\ndependencies that are added to the constructor. Any dependencies added to the branch will not be back propagated to the\nconstructor, allowing for dependency isolation. Because branches inherit but do not propagate, their dependency\nresolution defers until `Constructor.build` is called, when it is assumed all dependencies with customized\ninstantiations have been added.\n\nBecause `Injectables` require a special lifecycle `Constructor.branch` will accept any instantiation args that should be\npassed to the class when it is constructed.\n**Example**\n```py\nbranch = constructor.branch(Dependency)\n```\n',
    'author': 'Zech Zimmerman',
    'author_email': 'hi@zech.codes',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ZechCodes/Bevy',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from bevy.config.config import Config
from bevy.config.directory_resolver import DirectoryResolver
from bevy.config.json_loader import JSONLoader


__all__ = ["Config", "DirectoryResolver", "JSONLoader"]



from bevy.config.resolver import Reader
from json import loads, dumps
from typing import Any


class JSONLoader:
    file_types = ("json",)

    def __init__(self, reader: Reader):
        self.reader = reader

    def load(self) -> dict[str, Any]:
        return loads(self.reader.read())

    def save(self, data: dict[str, Any]):
        self.reader.save(dumps(data))
